#ifndef SCENE_HXX
#define SCENE_HXX

#include <vector>

#include "Ray.hxx"
#include "Camera.hxx"
#include "Light.hxx"
#include "Primitive.hxx"

class Scene
{
public:
  Scene();
  virtual ~Scene();
  
  // add another primitive to the scene
  void Add(Primitive* p);
  // add another light source to the scene
  void Add(Light* l);

  // intersect the ray with all objects in the scene
  virtual bool Intersect(Ray& ray);
  // find occluder
  virtual bool Occluded(Ray& ray);
  
  // trace the given ray and shade it, returnt the color of the shaded ray
  Vec3f RayTrace(Ray& ray);
    
  const Camera* camera() const; 
private:
  Scene(const Scene& );
  Scene& operator=(const Scene& );

  Camera* m_camera;
  // background color
  Vec3f m_bgColor;
  
  // primitives
  std::vector<Primitive*> m_primitives;
  // lights
  std::vector<Light*> m_lights;
};
#endif
